@@ -10,6 +10,7 @@ import spyctl.resources.machines as spyctl_machines
 import spyctl.resources.namespaces as spyctl_names
 import spyctl.resources.pods as spyctl_pods
 import spyctl.resources.nodes as spyctl_nodes
+import spyctl.resources.redflags as spyctl_redflags
 import spyctl.resources.policies as spyctl_policies
 import spyctl.spyctl_lib as lib
 
@@ -57,6 +58,8 @@ def handle_get(
         handle_get_nodes(name_or_id, st, et, output, **filters)
     elif resource == lib.PODS_RESOURCE:
         handle_get_pods(name_or_id, st, et, output, **filters)
+    elif resource == lib.REDFLAGS_RESOURCE:
+        handle_get_redflags(name_or_id, st, et, output, **filters)
     elif resource == lib.POLICIES_RESOURCE:
         handle_get_policies(name_or_id, output, **filters)
     else:
@@ -136,7 +139,7 @@ def handle_get_nodes(name_or_id, st, et, output: str, **filters: Dict):
     if name_or_id:
         nodes = filt.filter_obj(
             nodes,
-            [f"{lib.METADATA_FIELD}.{lib.METADATA_NAME_FIELD}", "id"],
+            [[lib.METADATA_FIELD, lib.METADATA_NAME_FIELD], lib.ID_FIELD],
             name_or_id,
         )
     if output != lib.OUTPUT_DEFAULT:
@@ -155,7 +158,11 @@ def handle_get_pods(name_or_id, st, et, output, **filters):
     pods = api.get_pods(*ctx.get_api_data(), clusters, (st, et))
     pods = filt.filter_pods(pods)
     if name_or_id:
-        pods = filt.filter_obj(pods, ["metadata.name", "id"], name_or_id)
+        pods = filt.filter_obj(
+            pods,
+            [[lib.METADATA_FIELD, lib.METADATA_NAME_FIELD], lib.ID_FIELD],
+            name_or_id,
+        )
     if output != lib.OUTPUT_DEFAULT:
         pods = spyctl_pods.pods_output(pods)
     cli.show(
@@ -165,6 +172,21 @@ def handle_get_pods(name_or_id, st, et, output, **filters):
     )
 
 
+def handle_get_redflags(name_or_id, st, et, output, **filters):
+    ctx = cfg.get_current_context()
+    flags = api.get_redflags(*ctx.get_api_data(), (st, et))
+    flags = filt.filter_redflags(flags, **filters)
+    if name_or_id:
+        flags = filt.filter_obj(flags, ["short_name", "id"], name_or_id)
+    if output != lib.OUTPUT_DEFAULT:
+        flags = spyctl_redflags.redflags_output(flags)
+    cli.show(
+        flags,
+        output,
+        {lib.OUTPUT_DEFAULT: spyctl_redflags.redflags_output_summary},
+    )
+
+
 def handle_get_fingerprints(name_or_id, st, et, output, **filters):
     ctx = cfg.get_current_context()
     machines = api.get_machines(*ctx.get_api_data())
@@ -181,15 +203,15 @@ def handle_get_fingerprints(name_or_id, st, et, output, **filters):
         cont_fprint_grps = filt.filter_obj(
             cont_fprint_grps,
             [
-                f"{lib.METADATA_FIELD}.{lib.IMAGE_FIELD}",
-                f"{lib.METADATA_FIELD}.{lib.IMAGEID_FIELD}",
+                [lib.METADATA_FIELD, lib.IMAGE_FIELD],
+                [lib.METADATA_FIELD, lib.IMAGEID_FIELD],
             ],
             name_or_id,
         )
         svc_fprint_grps = filt.filter_obj(
             svc_fprint_grps,
             [
-                f"{lib.METADATA_FIELD}.{lib.CGROUP_FIELD}",
+                [lib.METADATA_FIELD, lib.CGROUP_FIELD],
             ],
             name_or_id,
         )
@@ -215,8 +237,8 @@ def handle_get_policies(name_or_id, output, **filters):
         policies = filt.filter_obj(
             policies,
             [
-                f"{lib.METADATA_FIELD}.{lib.NAME_FIELD}",
-                f"{lib.METADATA_FIELD}.{lib.METADATA_UID_FIELD}",
+                [lib.METADATA_FIELD, lib.NAME_FIELD],
+                [lib.METADATA_FIELD, lib.METADATA_UID_FIELD],
             ],
             name_or_id,
         )
