@@ -14,21 +14,15 @@ def pods_output_summary(pods: List[Dict]) -> str:
     headers = [
         "NAME",
         "UID",
+        "CLUSTER",
         "READY",
         "LAST_STATUS_SEEN",
         "AGE",
         "LAST_SEEN_BY_SPYDERBAT",
         "SPYDERBAT_STATUS",
     ]
-    clusters: Dict[str, List] = {}  # cluster_uid -> list of pods
+    data = []
     for pod in pods:
-        cluster_name = pod.get("cluster_name")
-        if not cluster_name:
-            print(pod)
-            continue
-        cluster_key = f"{pod['cluster_name']} - {pod['cluster_uid']}"
-        clusters.setdefault(cluster_key, [])
-        clusters[cluster_key].append(pod)
         phase = pod["k8s_status"]["phase"]
         if phase == "Running":
             pod_status[pod["id"]] = "1/1"
@@ -36,34 +30,13 @@ def pods_output_summary(pods: List[Dict]) -> str:
             pod_status[pod["id"]] = "0/1"
         else:
             pod_status[pod["id"]] = "N/A"
-    output = ""
-    if len(clusters) == 1:
-        data = []
-        for pod in next(iter(clusters.values())):
-            data.append(pod_summary_data(pod))
-        output = tabulate(
-            sorted(data, key=lambda x: [x[6], x[0], _to_timestamp(x[5])]),
-            headers=headers,
-            tablefmt="plain",
-        )
-    elif len(clusters) > 1:
-        output = []
-        for cluster, pod_list in clusters.items():
-            output.append(cluster)
-            data = []
-            for pod in pod_list:
-                data.append(pod_summary_data(pod))
-            output.append(
-                tabulate(
-                    sorted(
-                        data, key=lambda x: [x[6], x[0], _to_timestamp(x[5])]
-                    ),
-                    headers,
-                    tablefmt="plain",
-                )
-                + "\n"
-            )
-        output = "\n".join(output)
+    for pod in pods:
+        data.append(pod_summary_data(pod))
+    output = tabulate(
+        sorted(data, key=lambda x: [x[2], x[7], x[0], _to_timestamp(x[6])]),
+        headers=headers,
+        tablefmt="plain",
+    )
     return output + "\n"
 
 
@@ -75,9 +48,13 @@ def pod_summary_data(pod: Dict) -> List:
     creation_timestamp = zulu.parse(
         pod[lib.METADATA_FIELD]["creationTimestamp"]
     )
+    cluster = pod.get("cluster_name") or pod.get("cluster_uid")
+    if not cluster:
+        cluster = "N/A"
     rv = [
         pod[lib.METADATA_FIELD][lib.METADATA_NAME_FIELD],
         pod["id"],
+        cluster,
         pod_status[pod["id"]],
         pod["k8s_status"]["phase"],
         f"{(zulu.now() - creation_timestamp).days}d",
@@ -90,11 +67,11 @@ def pod_summary_data(pod: Dict) -> List:
 
 def pods_output(pods: List[Dict]) -> Dict:
     if len(pods) == 1:
-        return pod_output(pods[0])
+        return pods[0]
     elif len(pods) > 1:
         return {
             lib.API_FIELD: lib.API_VERSION,
-            lib.ITEMS_FIELD: [pod_output(pod) for pod in pods],
+            lib.ITEMS_FIELD: pods,
         }
     else:
         return {}
