@@ -8,7 +8,7 @@ import time
 from base64 import urlsafe_b64encode as b64url
 from datetime import datetime
 from pathlib import Path
-from typing import Dict, Iterable, List, Tuple, Union
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 from uuid import uuid4
 
 import click
@@ -28,6 +28,9 @@ class Aliases:
         plural_match = __o == self.name_plural if self.name_plural else False
         return __o in self.aliases or __o == self.name or plural_match
 
+    def __str__(self) -> str:
+        return self.name_plural or self.name
+
 
 APP_NAME = "spyctl"
 WARNING_MSG = "is_warning"
@@ -63,6 +66,7 @@ MACHINES_RESOURCE = Aliases(
 )
 NODES_RESOURCE = Aliases(["nodes", "node"], "node", "nodes")
 PODS_RESOURCE = Aliases(["pods", "pod"], "pod", "pods")
+REDFLAGS_RESOURCE = Aliases(["redflags", "redflag"], "redflag", "redflags")
 FINGERPRINTS_RESOURCE = Aliases(
     [
         "fingerprints",
@@ -109,6 +113,7 @@ GET_RESOURCES: List[str] = [
     NODES_RESOURCE.name_plural,
     PODS_RESOURCE.name_plural,
     POLICIES_RESOURCE.name_plural,
+    REDFLAGS_RESOURCE.name_plural,
 ]
 VAL_RESOURCES: List[str] = [
     BASELINES_RESOURCE.name,
@@ -144,6 +149,24 @@ class GetResourcesParam(click.ParamType):
         ]
 
 
+class LabelParam(click.ParamType):
+    def convert(
+        self,
+        value: Any,
+        param: Optional[click.Parameter],
+        ctx: Optional[click.Context],
+    ) -> Any:
+        rv = label_input_to_dict()
+        if rv is None:
+            self.fail("Invalid label input", param, ctx)
+        return value
+
+
+# Spyderbat Schema Prefix'
+SCHEMA_FIELD = "schema"
+EVENT_REDFLAG_PREFIX = "event_redflag"
+EVENT_OPSFLAG_PREFIX = "event_opsflag"
+
 # Resource Kinds
 POL_KIND = "SpyderbatPolicy"
 BASELINE_KIND = "SpyderbatBaseline"
@@ -172,7 +195,7 @@ S_HIGH = "high"
 S_MED = "medium"
 S_LOW = "low"
 S_INFO = "info"
-ALLOWED_SEVERITIES = {S_CRIT, S_HIGH, S_MED, S_LOW, S_INFO}
+ALLOWED_SEVERITIES = [S_CRIT, S_HIGH, S_MED, S_LOW, S_INFO]
 
 # Config
 CURR_CONTEXT_FIELD = "current-context"
@@ -187,6 +210,8 @@ ORG_FIELD = "organization"
 POD_FIELD = "pod"
 CLUSTER_FIELD = "cluster"
 NAMESPACE_FIELD = "namespace"
+NAMESPACE_LABELS_FIELD = "namespace-labels"
+POD_LABELS_FIELD = "pod-labels"
 MACHINES_FIELD = "machines"
 DEFAULT_API_URL = "https://api.spyderbat.com"
 
@@ -662,6 +687,59 @@ class CustomCommand(click.Command):
             formatter.write_text(epilog)
 
 
+class ArgumentParametersCommand(CustomCommand):
+    argument_value_parameters = []
+    argument_name = ""
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.base_param_count = len(self.params)
+        self.argument_value = self.argument_name
+
+    def parse_args(self, ctx, args: List[str]) -> List[str]:
+        args_cpy = args.copy()
+        parser = self.make_parser(ctx)
+        parser.ignore_unknown_options = True
+        opts, args_cpy, param_order = parser.parse_args(args=args_cpy)
+        for param in self.get_params(ctx):
+            if param.name == self.argument_name:
+                try:
+                    param.handle_parse_result(ctx, opts, args_cpy)
+                except Exception:
+                    pass
+                break
+        argument_value = ctx.params.get(self.argument_name)
+        if argument_value:
+            for obj in self.argument_value_parameters:
+                for value_option in obj[self.argument_name]:
+                    if argument_value == value_option:
+                        self.argument_value = str(value_option)
+                        for arg_maker in obj["args"]:
+                            # single use, parse args twice will make dupes
+                            arg_maker(self)
+                        break
+        return super().parse_args(ctx, args)
+
+    def format_options(self, ctx, formatter):
+        """Writes all the options into the formatter if they exist."""
+        opts = []
+        specif_opts = []
+        for i, param in enumerate(self.get_params(ctx)):
+            rv = param.get_help_record(ctx)
+            if rv is not None:
+                if i < self.base_param_count:
+                    opts.append(rv)
+                else:
+                    specif_opts.append(rv)
+
+        if opts:
+            with formatter.section("Options"):
+                formatter.write_dl(opts)
+        if specif_opts:
+            with formatter.section(f"Options for {self.argument_value}"):
+                formatter.write_dl(specif_opts)
+
+
 def try_log(*args, **kwargs):
     try:
         if kwargs.pop(WARNING_MSG, False):
@@ -738,10 +816,152 @@ def selectors_to_filters(resource: Dict, **filters) -> Dict:
         return filters
     rv: Dict = copy.deepcopy(spec.get(CONT_SELECTOR_FIELD, {}))
     rv.update(copy.deepcopy(spec.get(SVC_SELECTOR_FIELD, {})))
+    namespace_labels = copy.deepcopy(
+        spec.get(NAMESPACE_SELECTOR_FIELD, {}).get(MATCH_LABELS_FIELD, {})
+    )
+    if namespace_labels:
+        rv.update({NAMESPACE_LABELS_FIELD: namespace_labels})
+    pod_labels = copy.deepcopy(
+        spec.get(POD_SELECTOR_FIELD, {}).get(MATCH_LABELS_FIELD, {})
+    )
+    if pod_labels:
+        rv.update({POD_LABELS_FIELD: pod_labels})
     rv.update(filters)
     return rv
 
 
+def label_input_to_dict(input: Union[str, List[str], Dict]) -> Optional[Dict]:
+    in_str = " in "
+    notin_str = " notin "
+
+    def parse_str_input(inp: str) -> Optional[Dict]:
+        def parse_only_key(key_inp: str) -> Optional[Dict]:
+            key_inp = key_inp.strip(" ")
+            if " " in key_inp:
+                return None
+            return {key_inp: "*"}
+
+        def parse_equality_based(eq_inp: str) -> Optional[Dict]:
+            rv = {}
+            kv_pairs = eq_inp.split(",")
+            for pair in kv_pairs:
+                try:
+                    k, v = pair.split("=")
+                    k = k.strip(" ")
+                    v = v.strip(" ")
+                    rv[k] = v
+                except Exception:
+                    only_key = parse_only_key(pair)
+                    if not only_key:
+                        try_log(
+                            f"{pair} is an invalid format. Use 'key=value' or"
+                            " only 'key'",
+                            is_warning=True,
+                        )
+                        return None
+                    rv.update(only_key)
+            if not rv:
+                return None
+            return rv
+
+        def parse_set_based(set_inp) -> Optional[Dict]:
+            rv = {}
+            import re
+
+            # split input on commas not in parenthesis
+            pat = re.compile(r",(?![^(]*\))")
+            for set_str in re.split(pat, set_inp):
+                if in_str in set_str:
+                    try:
+                        k, s = set_str.split(in_str)
+                        s = s.replace("(", "").replace(")", "").split(",")
+                        s = [
+                            value.strip(" ") for value in s if value.strip(" ")
+                        ]
+                        if not s:
+                            try_log(
+                                f"{set_str} cannot contain an empty",
+                                is_warning=True,
+                            )
+                            return None
+                        if len(s) == 1:
+                            s = s[0]
+                        rv[k] = s
+                    except Exception:
+                        try_log(
+                            f"{set_str} is an invalid format use 'key in"
+                            " (value1,value2)' or only 'key'",
+                            is_warning=True,
+                        )
+                        return None
+                else:
+                    only_key = parse_only_key(set_str)
+                    if not only_key:
+                        try_log(
+                            f"{set_str} is an invalid format use 'key in"
+                            " (value1,value2)' or only 'key'",
+                            is_warning=True,
+                        )
+                        return None
+                    rv.update(only_key)
+            if not rv:
+                return None
+            return rv
+
+        rv = None
+        if "=" in inp:
+            rv = parse_equality_based(inp)
+        elif in_str in inp:
+            if notin_str in inp:
+                try_log("notin not supported", is_warning=True)
+                return None
+            rv = parse_set_based(inp)
+        elif notin_str in inp:
+            if notin_str in inp:
+                try_log("notin not supported", is_warning=True)
+                return None
+        else:
+            rv = {}
+            only_keys = inp.split(",")
+            for key in only_keys:
+                parsed_key = parse_only_key(key)
+                if parsed_key is None:
+                    return None
+                rv.update(parsed_key)
+        if not rv:
+            return None
+        return rv
+
+    rv = {}
+    if isinstance(input, str):
+        parsed = parse_str_input(input)
+        if not parsed:
+            return None
+        rv.update(parsed)
+    elif isinstance(input, list):
+        for item in input:
+            if not isinstance(item, str):
+                try_log(
+                    f"label list contains items other than a string. {input}",
+                    is_warning=True,
+                )
+                return None
+            parsed = parse_str_input(item)
+            if not parsed:
+                return None
+            rv.update(parsed)
+    elif isinstance(input, dict):
+        if not input:
+            return None
+        rv.update(input)
+    else:
+        try_log("label input is not str, list, or dict", is_warning=True)
+        return None
+    if not rv:
+        return None
+    return rv
+
+
 def make_uuid():
     return b64url(uuid4().bytes).decode("ascii").strip("=")
 
